"""Feed Manager for NSW Rural Fire Service Incidents feed."""
from typing import List, Dict, Tuple, Callable, Awaitable

from aio_geojson_client.feed_manager import FeedManagerBase
from aio_geojson_client.status_update import StatusUpdate
from aiohttp import ClientSession

from .feed import GeoJsonQueryFeed


class GeoJsonQueryFeedManager(FeedManagerBase):

    def __init__(self,
                 websession: ClientSession,
                 endpoint: str,
                 generate_callback: Callable[[str], Awaitable[None]],
                 update_callback: Callable[[str], Awaitable[None]],
                 remove_callback: Callable[[str], Awaitable[None]],
                 coordinates: Tuple[float, float],
                 filter_radius: float = None,
                 filter_criteria: List[List[str]] = None,
                 mappings: Dict[str, str] = None,
                 status_callback: Callable[[StatusUpdate],
                                           Awaitable[None]] = None):
        """Initialize the NSW Rural Fire Services Feed Manager."""
        feed = GeoJsonQueryFeed(
            websession,
            endpoint,
            coordinates,
            filter_radius=filter_radius,
            filter_criteria=filter_criteria,
            mappings=mappings)
        super().__init__(feed,
                         generate_callback,
                         update_callback,
                         remove_callback,
                         status_async_callback=status_callback)



import logging
from typing import List, Optional, Tuple, Dict
from datetime import datetime

from aio_geojson_client.feed import GeoJsonFeed
from aiohttp import ClientSession
from geojson import FeatureCollection

from .feed_entry import GeoJsonQueryEntry

from functools import reduce

_LOGGER = logging.getLogger(__name__)


class GeoJsonQueryFeed(
        GeoJsonFeed[GeoJsonQueryEntry]):

    def __init__(self,
                 websession: ClientSession,
                 endpoint: str,
                 home_coordinates: Tuple[float, float],
                 filter_radius: float = None,
                 filter_criteria: List[List[str]] = None,
                 mappings: Dict[str, str] = None):
        """Initialise this service."""
        super().__init__(websession,
                         home_coordinates,
                         endpoint,
                         filter_radius=filter_radius)
        self._filter_criteria = filter_criteria
        self._mappings = mappings

    def __repr__(self):
        """Return string representation of this feed."""
        return '<{}(home={}, url={}, radius={}, criteria={})>'.format(
            self.__class__.__name__, self._home_coordinates, self._url,
            self._filter_radius, self._filter_criteria)

    def _new_entry(self, home_coordinates: Tuple[float, float], feature,
                   global_data: Dict) -> GeoJsonQueryEntry:
        """Generate a new entry."""
        return GeoJsonQueryEntry(home_coordinates, feature, self._mappings)

    # TODO Allow for 'and' filters -- at this time, these are 'or' only
    # TODO At this point, this is a bit confusing because our gt/lt tests
    # end up being inclusive(!)
    def _filter_entries(self,
                        entries: List[GeoJsonQueryEntry]) \
            -> List[GeoJsonQueryEntry]:

        def _compare(v1, v2, op):
            if op == '==':
                return v1 == v2
            elif op == '!=':
                return v1 != v2
            elif op == '>':
                return float(v1) > float(v2)
            elif op == '<':
                return float(v1) < float(v2)
            else:
                return False # TODO

        """Filter the provided entries."""
        filtered_entries = super()._filter_entries(entries)
        if self._filter_criteria:
            filtered_entries = list(filter(lambda entry:
                                    reduce((lambda truthiness, filter:
                                                truthiness or _compare(entry.criteria(filter[0]), filter[2], filter[1])),
                                             self._filter_criteria,
                                             False),
                                    filtered_entries))
        return filtered_entries

    def _extract_last_timestamp(
            self,
            feed_entries: List[GeoJsonQueryEntry]) \
            -> Optional[datetime]:
        """Determine latest (newest) entry from the filtered feed."""
        if feed_entries:
            dates = sorted(filter(
                None, [entry.publication_date for entry in feed_entries]),
                reverse=True)
            return dates[0]
        return None

    def _extract_from_feed(self, feed: FeatureCollection) -> Optional[Dict]:
        """Extract global metadata from feed."""
        return None



"""NSW Rural Fire Service Incidents feed entry."""
import pytz
import calendar
from datetime import datetime
from time import strptime

import logging
import re
from typing import Optional, Dict, Tuple
from aio_geojson_client.feed_entry import FeedEntry
from geojson import Feature

from .consts import ATTRIBUTION, CUSTOM_ATTRIBUTE

_LOGGER = logging.getLogger(__name__)


class GeoJsonQueryEntry(FeedEntry):
    """NSW Rural Fire Service Incidents feed entry."""


    def __init__(self,
                 home_coordinates: Tuple[float, float],
                 feature: Feature,
                 mappings: Dict[str, str]):
        """Initialise this service."""
        super().__init__(home_coordinates, feature)
        self._mappings = mappings


    @property
    def attribution(self) -> Optional[str]:
        """Return the attribution of this entry."""
        return ATTRIBUTION


    @property
    def title(self) -> str:
        """Return the title of this entry."""
        title = self._parametric_search('title')
        if title: return title
        title = self._search_in_properties('title')
        if title: return title
        title = self._search_in_properties('details')
        if title: return title
        return None


    @property
    def external_id(self) -> str:
        """Return the external id of this entry."""
        id = self._parametric_search('id')
        if id: return id
        id = self._search_in_properties('id')
        if id: return id
        id = self._search_in_properties('guid')
        if id: return id
        return None


    @property
    def publication_date(self) -> datetime:
        """Return the publication date of this entry."""
        publication_date = self._parametric_search('date')
        if not publication_date:
            publication_date = self._search_in_properties('time')
        if not publication_date:
            publication_date = self._search_in_properties('date')
        if publication_date:
            # So, what kind of date are we facing?
            if self._mappings and 'dateformat' in self._mappings:
                date_format = self._mappings['dateformat']
                if date_format == 'seconds':
                    publication_date = datetime.fromtimestamp(publication_date)
                elif date_format == 'milliseconds':
                    publication_date = datetime.fromtimestamp(publication_date / 1000)
                else:
                # Parse the date. Example: 15/09/2018 9:31:00 AM
                    date_struct = strptime(publication_date, date_format)
                    publication_date = datetime.fromtimestamp(calendar.timegm(
                        date_struct), tz=pytz.utc)
            else:
                    date_struct = strptime(publication_date, "%d/%m/%Y %I:%M:%S %p")
                    publication_date = datetime.fromtimestamp(calendar.timegm(
                        date_struct), tz=pytz.utc)
        return publication_date


    def _parametric_search(self, search) -> str:
        if self._mappings and search in self._mappings:
            criteria = self._mappings[search]
            unpacked = criteria.split('~~', 1)
            if len(unpacked) > 1:
                varname, formula = unpacked
                value = self._search_in_properties(varname)
                match = re.search(formula.format(CUSTOM_ATTRIBUTE), value)
                if match:
                    return match.group(CUSTOM_ATTRIBUTE)
            else:
                varname = unpacked[0]
                return self._search_in_properties(varname)
        return None


    @property
    def description(self) -> str:
        description = self._parametric_search('description')
        if description: return description
        description = self._search_in_properties('description')
        if description: return description
        description = self._search_in_properties('details')
        if description: return description
        return None


    @property
    def location(self) -> str:
        """Return the location of this entry."""
        return self._parametric_search('location')


    @property
    def area(self) -> str:
        """Return the council area of this entry."""
        return self._parametric_search('area')


    @property
    def status(self) -> str:
        """Return the status of this entry."""
        return self._parametric_search('status')


    @property
    def type(self) -> str:
        """Return the type of this entry."""
        return self._parametric_search('type')

    @property
    def size(self) -> str:
        """Return the size of this entry."""
        return self._parametric_search('size')

    @property
    def agency(self) -> str:
        """Return the responsible agency of this entry."""
        return self._parametric_search('agency')

    def criteria(self, name) -> str:
        return self._search_in_properties(name)



import os
from setuptools import setup


def get_readme_content():
    return open(os.path.join(os.path.dirname(__file__), 'README.md')).read()


setup(
    name = 'aio-geojson-query',
    version = '0.0.1',
    description = 'A generalized client for aio-geojson-client',
    author = 'Chris F Ravenscroft',
    author_email = 'chris@voilaweb.com',
    url = 'https://github.com/exxamalte/python-aio-geojson-client',
    license = 'MIT',
    long_description = get_readme_content(),
    long_description_content_type="text/markdown",
    packages = ['aio_geojson_query',],
    classifiers = [
        'License :: OSI Approved :: MIT License',
        'Development Status :: 4 - Beta',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    install_requires = [
        'aiodns', 'aiohttp>=3.5.4',
        'aio_geojson_client>=0.13',
        'pytz>=2019.01'
        ],
)
